@@ -4,10 +4,9 @@ import os
 import logging
 import threading
 import configparser
-import uuid
+import time
 from confluent_kafka import Consumer, TopicPartition, Producer, OFFSET_END
 from confluent_kafka import KafkaError, KafkaException
-from confluent_kafka.admin import AdminClient
 
 class TopicUnknown(Exception):
     """Exception raised when the timeout is expired on operation"""
@@ -48,8 +47,8 @@ class TopicChecker:
 class Broker:
     """
     Represent the abstraction of the protocol 
-    which the k2eg client talk with the k2eg g
-    ateway
+    which the k2eg client talk with the k2eg 
+    gateway
 
     ...
 
@@ -58,15 +57,14 @@ class Broker:
 
     Methods
     -------
-    get_reply_topic()
-        Return the string of the reply topic
     """
     def __init__(
         self, 
-        environment_id: str, 
-        group_name: str = str(uuid.uuid4())[:8],
-        app_name:str =  "ke2g-app",
-        app_instance_unique_id:str = "1"):
+        environment_id: str,
+        app_name:str,
+        group_name: str = None,
+        app_instance_unique_id:str = "1",
+        startup_tout: int = 10):
         """
         Parameters
         ----------
@@ -77,18 +75,20 @@ class Broker:
             Is the group name to distribuite the data from different same 
             client instance
         """
+        if group_name is None:
+            group_name = app_name
         enviroment_set: str = 'DEFAULT'
         self.__initialized=False
         # Get the current directory of the script
         current_configuration_dir = os.getenv(
             'K2EG_PYTHON_CONFIGURATION_PATH_FOLDER', 
-            os.path.dirname(os.path.realpath(__file__))
+            os.path.dirname(os.path.realpath(__file__))+"/environment"
         ) 
         enable_kafka_debug = os.getenv(
             'K2EG_PYTHON_ENABLE_KAFKA_DEBUG_LOG', 
             'false'
         ).lower in ("yes", "true", "t", "1")
-        enable_kafka_debug = True
+
         self.__enviroment_set = enviroment_set
         # Create a new ConfigParser object
         self.__config = configparser.ConfigParser()
@@ -97,7 +97,7 @@ class Broker:
         self.__config.read(
             os.path.join(
             current_configuration_dir, 
-            f'environment/{environment_id}.ini')
+            f'{environment_id}.ini')
             )
 
         self.__check_config(app_name)
@@ -107,14 +107,13 @@ class Broker:
                 self.__enviroment_set, 'kafka_broker_url'
                 ), 
             'group.id': group_name,
-            'group.instance.id': group_name+'_'+app_instance_unique_id,
+            'group.instance.id': app_name+'_'+app_instance_unique_id,
             'auto.offset.reset': 'latest',
             'enable.auto.commit': 'false',
-            'allow.auto.create.topics': 'true',
             'topic.metadata.refresh.interval.ms': '60000'
         }
-        if enable_kafka_debug:
-            config_consumer['debug'] = 'consumer,cgrp,topic,fetch'
+        if enable_kafka_debug is True:
+            config_consumer['debug'] = 'consumer,cgrp,topic' #fetch
 
         self.__consumer = Consumer(config_consumer)
         config_producer = {
@@ -124,34 +123,38 @@ class Broker:
             #'debug': 'consumer,cgrp,topic,fetch',
         }
         self.__producer = Producer(config_producer)
-        self.__admin = AdminClient({'bootstrap.servers': self.__config.get(
-                self.__enviroment_set, 'kafka_broker_url'
-                )})
         self.__reply_topic = app_name + '-reply'
         self.__reply_partition_assigned = threading.Event()
         self.__subribed_topics = [self.__reply_topic]
+
+        # start consuming from reply topic
         self.__consumer.subscribe(self.__subribed_topics, on_assign=self.__on_assign)
         self.__reply_topic_joined = False
         self.__topic_checker = TopicChecker()
         # wait for consumer join the partition
-        self.__consumer.poll(0.1)
-        self.__reply_partition_assigned.wait()
+        self.wait_for_reply_available(startup_tout)
+        
+
+    def __manage_old_state(self):
+        self.__consumer.poll(1.0)
+        assigned_topics = self.__consumer.assignment()
+        for t in assigned_topics:
+            logging.debug(f'latest assigned topic: {t}')
+        self.__consumer.unsubscribe()
 
     # point always to the end of the topic
     def __on_assign(self, consumer, partitions):
-        logging.debug(f"Joined partition {partitions}")
         for p in partitions:
+            logging.debug(f"Joined topic {p.topic} with partition {p.partition}")
             if p.topic == self.__reply_topic:
                 self.__reply_topic_joined = True
                 self.__reply_partition_assigned.set()
-                #low, high = consumer.get_watermark_offsets(p)
-                if p.offset==-1:
-                    p.offset = OFFSET_END
-        positions = consumer.position(partitions)
-        logging.debug('assign: {}'.format(' '.join(map(str, partitions))))
-        logging.debug('position: {}'.format(' '.join(map(str, positions))))
+            #low, high = consumer.get_watermark_offsets(p)
+            if p.offset==-1:
+                logging.debug(f'set new offset for {p.topic}')
+                p.offset = OFFSET_END
         consumer.assign(partitions)
-        print('Assigned partition')
+
 
     def __check_config(self, app_name):
         if not self.__config.has_option(self.__enviroment_set, 'kafka_broker_url'):
@@ -163,12 +166,16 @@ class Broker:
     def get_reply_topic(self):
         return self.__reply_topic
 
-    def wait_for_reply_available(self):
+    def wait_for_reply_available(self, timeout):
         """ Wait untile the consumer has joined the reply topic
         """
-        self.__reply_partition_assigned.wait()
-        if self.__reply_topic_joined is False:
-                raise TopicUnknown(f"{self.__reply_partition} unknown")
+        start_time = time.time()
+        end_time = start_time + timeout
+        while self.__reply_partition_assigned.wait(1) is False:
+            if time.time() > end_time:
+                raise TimeoutError("Function timed out")
+            logging.debug("waiting for reply topic to join")
+            self.__consumer.poll(1)
 
     def reset_reply_topic_to_ts(self, timestamp):
         self.reset_topic_offset_in_time(self.__reply_topic, timestamp)
@@ -193,7 +200,6 @@ class Broker:
         except KafkaException as e:
             logging.error(e)
        
-    
     def get_next_message(self, timeout = 0.1):
         message = self.__consumer.poll(timeout)
         # give a chanche to update metadata ofr pending topics
@@ -213,6 +219,7 @@ class Broker:
             raise ValueError(
                 f'The topic name {self.__reply_topic} cannot be used'
                 )
+        logging.debug(f'Start consuming from topic: {new_topic}')
         if new_topic not in self.__subribed_topics:
             self.__subribed_topics.append(new_topic)
             self.__consumer.subscribe(
@@ -240,51 +247,45 @@ class Broker:
         )
         self.__producer.flush()
 
-    def send_get_command(self, pv_name, protocol, reply_id):
+    def send_get_command(self, pv_uri, reply_id):
         get_json_msg = {
             "command": "get",
             "serialization": "msgpack",
-            "protocol": protocol.lower(),
-            "pv_name": pv_name,
+            "pv_name": pv_uri,
             "reply_topic": self.__reply_topic,
             "reply_id": reply_id
         }
         self.send_command(json.dumps(get_json_msg))
 
-    def send_start_monitor_command(self, pv_name, protocol, pv_reply_topic, reply_id):
-        # ensure topic exists
-        # self.__create_topics(pv_reply_topic)
-
+    def send_start_monitor_command(self, pv_uri, pv_reply_topic, reply_id):
         # send command
         monitor_json_msg = {
             "command": "monitor",
             "serialization": "msgpack",
-            "protocol": protocol.lower(),
-            "pv_name": pv_name,
+            "pv_name": pv_uri,
+            "activate": True,
             "reply_topic": self.__reply_topic,
             "reply_id": reply_id,
-            "activate": True,
             "monitor_dest_topic": pv_reply_topic
         }
         self.send_command(json.dumps(monitor_json_msg))    
-
-    def send_stop_monitor_command(self, pv_name, pv_reply_topic, reply_id):
-        monitor_json_msg = {
-            "command": "monitor",
-            "serialization": "msgpack",
-            "pv_name": pv_name,
-            "reply_topic": self.__reply_topic,
-            "reply_id": reply_id,
-            "activate": False,
-            "monitor_dest_topic": pv_reply_topic
-        }
-        self.send_command(json.dumps(monitor_json_msg))
     
-    def send_put_command(self, pv_name: str, value: any, protocol:str, reply_id: str):
+    def send_start_monitor_command_many(self, pv_uri_list, reply_id):
+        # send command
+        monitor_json_msg = {
+            "command": "multi-monitor",
+            "serialization": "msgpack",
+            "activate": True,
+            "pv_name_list": pv_uri_list,
+            "reply_topic": self.__reply_topic,
+            "reply_id": reply_id
+        }
+        self.send_command(json.dumps(monitor_json_msg))  
+
+    def send_put_command(self, pv_uri: str, value: any, reply_id: str):
         put_value_json_msg = {
             "command": "put",
-            "protocol": protocol,
-            "pv_name": pv_name,
+            "pv_name": pv_uri,
             "value": str(value),
             "reply_topic": self.__reply_topic,
             "reply_id": reply_id,
