@@ -8,7 +8,7 @@ from confluent_kafka import KafkaError
 from typing import Callable
 from k2eg.broker import Broker
 
-_protocol_regex = r"(pva?|ca)://([-\w.]+(/[-\w./]*)*:.*)"
+_protocol_regex = r"^(pva?|ca)://((?:[A-Za-z0-9-_:]+(?:\.[A-Za-z0-9-_]+)*))$"
 
 def _filter_pv_uri(uri: str):
     match = re.match(_protocol_regex, uri)
@@ -35,8 +35,14 @@ class dml:
     def __init__(
             self, 
             environment_id: str, 
-            app_name: str = str(uuid.uuid1())):
-        self.__broker = Broker(environment_id, app_name, app_name)
+            app_name: str):
+        if app_name is None:
+            raise ValueError(
+                "The app name is mandatory"
+            )
+        self.__broker = None
+        self.__thread = None
+        self.__broker = Broker(environment_id, app_name)
         self.__lock = rwlock.RWLockFairD()
         self.__reply_partition_assigned = threading.Event()
         
@@ -132,22 +138,19 @@ class dml:
                 was_a_reply = False
                 #msg_id could be a reply id or pv name
                 msg_id, decoded_message = self.__decode_message(message)
-                logging.debug(f"received with offset [{msg_id}|{message.offset()}]")
                 if msg_id is None or decoded_message is None:
                     continue
                 with self.reply_wait_condition:
                     was_a_reply = msg_id in self.reply_message
                     if was_a_reply is True:
+                        logging.debug(f"received reply on topic {message.topic()}")
                         self.reply_message[msg_id] = decoded_message
                         self.reply_wait_condition.notifyAll()
                     elif msg_id in self.__monitor_pv_handler:
+                        logging.debug(f"received event on topic {message.topic()}")
                         self.__monitor_pv_handler[msg_id](
                             msg_id, decoded_message[msg_id]
                             )
-                        logging.debug(
-                            'read message sent '+
-                            f'to {self.__monitor_pv_handler[msg_id]} handler'
-                        )
                 self.__broker.commit_current_fetched_message()
 
     def parse_pv_url(self, pv_url):
@@ -204,8 +207,7 @@ class dml:
             self.reply_message[new_reply_id] = None
             # send message to k2eg
             self.__broker.send_get_command(
-                pv_name,
-                protocol.lower(),
+                pv_url,
                 new_reply_id
             )
             while(not fetched):
@@ -249,9 +251,8 @@ class dml:
             self.reply_message[new_reply_id] = None
             # send message to k2eg
             self.__broker.send_put_command(
-                pv_name,
+                pv_url,
                 value,
-                protocol.lower(),
                 new_reply_id
             )
             while(not fetched):
@@ -295,8 +296,7 @@ class dml:
             # send message to k2eg from activate (only for last topics) 
             # monitor(just in case it is not already activated)
             self.__broker.send_start_monitor_command(
-                pv_name,
-                protocol,
+                pv_url,
                 self.__normalize_pv_name(pv_name),
                 new_reply_id,
             )
@@ -315,12 +315,12 @@ class dml:
                     self.__monitor_pv_handler[pv_name] = handler
                     self.__broker.add_topic(self.__normalize_pv_name(pv_name))
                     return result
-
-    def stop_monitor(self, pv_name: str, timeout: float = None):
-        """ Stop a new monitor for pv if it is not already activated
+        
+    def monitor_many(self, pv_uri_list: list[str], handler: Callable[[str, dict], None], timeout: float = None):  # noqa: E501
+        """ Add a new monitor for pv if it is not already activated
         Parameters
                 ----------
-                pv_name : str
+                pv_uri_list : list[str]
                     The name of the PV to monitor
                 handler: function
                     The handler to be called when a message is received
@@ -330,38 +330,52 @@ class dml:
                 False: otherwhise
         """
         fetched = False
-        self.__check_pv_name(pv_name)
+        for pv_uri in pv_uri_list:
+            protocol, pv_name = self.parse_pv_url(pv_uri)
+            if protocol.lower() not in ("pva", "ca"):
+                raise ValueError("The protocol need to be one of 'pva'  'ca'")
         new_reply_id = str(uuid.uuid1())
         with self.reply_wait_condition:
-            if pv_name not in self.__monitor_pv_handler:
-                logging.info(
-                    f"Monitor already stopped for pv {pv_name}")
-                return
+            filtered_list_pv_uri = []
             # init reply slot
-            self.reply_message[new_reply_id] = None
-            del self.__monitor_pv_handler[pv_name]
-            self.__broker.remove_topic(self.__normalize_pv_name(pv_name))
+            for pv_uri in pv_uri_list:
+                protocol, pv_name = self.parse_pv_url(pv_uri)
+                self.reply_message[new_reply_id] = None
+                if pv_name in self.__monitor_pv_handler:
+                    logging.info(
+                        f"Monitor already activate for pv {pv_name}")
+                    continue
+                filtered_list_pv_uri.append(pv_uri)
+            
+            if len(filtered_list_pv_uri)==0:
+                return
             # send message to k2eg from activate (only for last topics) 
             # monitor(just in case it is not already activated)
-            self.__broker.send_stop_monitor_command(
-                pv_name,
-                self.__normalize_pv_name(pv_name),
-                new_reply_id
+            self.__broker.send_start_monitor_command_many(
+                filtered_list_pv_uri,
+                new_reply_id,
             )
+
             while(not fetched):
                 op_res, result =  self.__wait_for_reply(new_reply_id, timeout)
                 if op_res == -2:
                     # raise timeout exception
                     raise OperationTimeout(
-                            f"Timeout during stop monitor operation for {pv_name}"
+                            f"Timeout during start monitor operation for {pv_name}"
                             )
                 elif op_res == -1:
                     continue
                 else:
+                    # all is gone ok i can register the handler and subscribe
+                    for pv_uri in filtered_list_pv_uri:
+                        protocol, pv_name = self.parse_pv_url(pv_uri)
+                        self.__monitor_pv_handler[pv_name] = handler
+                        self.__broker.add_topic(self.__normalize_pv_name(pv_name))
                     return result
-        
-
+                
     def close(self):
         self.__consume_data = False
-        self.__broker.close()
-        self.__thread.join()
\ No newline at end of file
+        if self.__broker is not None:
+            self.__broker.close()
+        if self.__thread is not None:
+            self.__thread.join()
\ No newline at end of file
